// Copyright 2000 by Kevin Atkinson under the terms of the LGPL

#include <ctime>

#include "writable_base.hh"
#include "file_util.hh"
#include "data_util.hh"
#include "file_exceps.hh"

namespace aspell {
  
  namespace writable_base {

    using namespace aspell::data_util; 

    void WritableBaseCode::update_file_date_info(fstream & f) {
      update_file_info(f);
      cur_file_date = get_modification_time(f);
    }

    void WritableBaseCode::load(const string & fn, 
				Config * config)
    {
      string f = fn;
      set_file_name(fn);
      fstream in;
    
      if (file_exists(fn)) {
      
	open_file_readlock(in, f);
	merge(in, f, config);
      
      } else if (fn.substr(fn.size()-suffix.size(),suffix.size()) 
		 == suffix) {
      
	compatibility_file_name = fn.substr(0,fn.size() - suffix.size());
	compatibility_file_name += compatibility_suffix;
	f = compatibility_file_name;
      
	try {
	  open_file_readlock(in, f);
	  merge(in, f, config);
	} catch (...) {
	  compatibility_file_name = "";
	  throw;
	}
      
      } else {
      
	throw CantReadFile(f);
      
      }

      update_file_date_info(in);
    }

    void WritableBaseCode::merge(const string & fn) {
      fstream in;
      open_file_readlock(in, fn);
      merge(in, fn);
    }

    void WritableBaseCode::update(fstream & in, const string & fn) {
      try {
      
	merge(in, fn);
	update_file_date_info(in);
      
      } catch (CantReadFile &) {
      
	if (compatibility_file_name.empty())
	  throw;
      }
    }
    
    void WritableBaseCode::save2(fstream & out, const string & f) {
      truncate_file(out, f);
      
      save(out,f);

      out.flush();
    }

    void WritableBaseCode::save_as(const string & fn) {
      compatibility_file_name = "";
      set_file_name(fn);
      fstream inout;
      open_file_writelock(inout, fn);
      save2(inout, fn);
      update_file_date_info(inout);
    }

    void WritableBaseCode::save(bool do_update) {
      fstream inout;
      bool prev_existed = open_file_writelock(inout, file_name());

      if (do_update
	  && prev_existed 
	  && get_modification_time(inout) > cur_file_date)
	update(inout, file_name());

      save2(inout, file_name());
      update_file_date_info(inout);
    
      if (compatibility_file_name.size() != 0) {
	remove_file(compatibility_file_name.c_str());
	compatibility_file_name = "";
      }
    
    }
  
  }
}
